/*
 * FAB Academy 2020
 * Rafeal Rebolleda // ESNE — Madrid, Spain.
 * 
 * This is the code that controls the UI and the machine behaviour
 * 
 */

#include <Adafruit_NeoPixel.h>

#include <Servo.h>
Servo servo;

// Constants won't change. Used here to set a pin number: 
const int MODE_SWITCH = 5;
      int MODE = 0; // 0: MANUAL; 1: AUTO.

// MANUAL MODE
// CW or CCW
const int CW_SWITCH = 2;
      int CW = 0;

// Pot
const int POT = A0;

// Reference LED
const int LED_NEO = 7;
Adafruit_NeoPixel pixel = Adafruit_NeoPixel(1, LED_NEO, NEO_GRB + NEO_KHZ800);

const int RAND_BUTTON = 3;
      int RAND_BUTTON_STATE = 1;
      int RAND_BUTTON_LAST_STATE = 1;
      uint32_t CURRENT_COLOR = pixel.Color(255, 0, 0);

// Strips
const int LED_STRIPS = 16;
Adafruit_NeoPixel strips = Adafruit_NeoPixel(28, LED_STRIPS, NEO_GRB + NEO_KHZ800);

// POT
int CURRENT_BRIGHTNESS = 10;
int TOP_BEGIN = 7;
int TOP_COUNT = 14;

// SPIN
const int SPIN_BUTTON = 4;
      int SPIN_BUTTON_STATE = 0;
      int SPIN_BUTTON_LAST_STATE = 0;


// AUTO MODE
const int PROG_BUTTON = 6;
      int PROG_BUTTON_STATE = 1;
      int PROG_BUTTON_LAST_STATE = 1;
      int CURRENT_PROGRAM = 1;

const int LED_1   =  10;
const int LED_2   =  11;
const int LED_3   =  12;

const int RUN_BUTTON = 8;
      int RUN_BUTTON_STATE = 0;
      int RUN_BUTTON_LAST_STATE = 0;
  
const int STOP_BUTTON = 9;
      int STOP_BUTTON_STATE = 0;
      int STOP_BUTTON_LAST_STATE = 0;

void setup() {
  
  Serial.begin(9600);
  Serial.println("INIT");

  
  // INPUTS
  pinMode(MODE_SWITCH, INPUT_PULLUP);
  
  pinMode(CW_SWITCH, INPUT_PULLUP);
  pinMode(POT, INPUT_PULLUP);
  pinMode(RAND_BUTTON, INPUT_PULLUP);
  pinMode(SPIN_BUTTON, INPUT_PULLUP);        
  pinMode(PROG_BUTTON, INPUT_PULLUP);
  pinMode(RUN_BUTTON, INPUT_PULLUP);
  pinMode(STOP_BUTTON, INPUT_PULLUP);  
  
  Serial.println("INPUTS CONFIGURED");
  
  // OUTPUTS

  servo.attach(16);
  servo.write(95); // STOP
  
  
  pixel.begin();
  strips.begin();
  strips.setBrightness(CURRENT_BRIGHTNESS);
  
  pinMode(LED_1, OUTPUT);
  pinMode(LED_2, OUTPUT);
  pinMode(LED_3, OUTPUT);

  Serial.println("OUTPUTS CONFIGURED");

  light_up();
  prog_led(1);
  
  Serial.println("FINISHED INIT");

}

void loop() {

  if (digitalRead(MODE_SWITCH) == HIGH) {
    MODE = 1;
    Serial.print("MODE = ");
    Serial.println(MODE);
  }
  else { 
    MODE = 0;
    Serial.print("MODE = ");
    Serial.println(MODE);
  }

  switch (MODE) {
      
      case 0: // MANUAL

        prog_led(0);
        pixel.setPixelColor(0, CURRENT_COLOR);
        pixel.show();

        handle_CW_switch();      
        handle_random_button();
        handle_pot();
        handle_spin();
        top_lights();
              
        break;

      case 1: // AUTO

        pixel.clear();
        pixel.show();

        strips.clear();
        strips.show();

        prog_led(CURRENT_PROGRAM);
        handle_program_change();

        handle_run_button();
        handle_stop_button();

        break;
    }
  
  
 
}



// MANUAL MODE ###################################################################################################### //
void handle_CW_switch(){
  if (digitalRead(CW_SWITCH) == HIGH) {
    CW = 1;
    Serial.print("CW = ");
    Serial.print(CW);
    Serial.println(" CLOCKWISE");
    
  }
  else { 
    CW = 0;
    Serial.print("CW = ");
    Serial.print(CW);
    Serial.println(" COUNTER-CLOCKWISE");
  }

}

void handle_pot(){

  if (false) {
    CURRENT_BRIGHTNESS = map(analogRead(POT), 0, 900, 255, 0);
  
    Serial.print("POT = ");
    Serial.println(CURRENT_BRIGHTNESS);
  
    strips.setBrightness(CURRENT_BRIGHTNESS);
    
  } else {
    
    if (analogRead(POT) < 850) {
      TOP_BEGIN = map(analogRead(POT), 0, 900, 20, 7);
      TOP_COUNT = 1;
      strips.clear();
      strips.show();
      
    } else {
      TOP_BEGIN = 7;
      TOP_COUNT = 14;
      strips.clear();
      strips.show();
    }
    
  }
  
}

void handle_random_button(){
  
  RAND_BUTTON_STATE = digitalRead(RAND_BUTTON);
  Serial.print("RAND_BUTTON_STATE: ");
  Serial.println(RAND_BUTTON_STATE);
  
  if ( RAND_BUTTON_STATE != RAND_BUTTON_LAST_STATE ) {

    if (RAND_BUTTON_STATE == LOW) {

      CURRENT_COLOR = pixel.Color(random(0,255), random(0,255), random(0,255));
    
      Serial.print("CURRENT COLOR: ");
      Serial.println(CURRENT_COLOR);
      
    }             
  }

  RAND_BUTTON_LAST_STATE = RAND_BUTTON_STATE;
  
}

void handle_spin() {
  
  if (digitalRead(SPIN_BUTTON) == LOW) {
    if (CW) {
      servo.write(0);    
    } else {
      servo.write(180);
    }
   
  } else {
    servo.write(95);  
  }
  
}

// AUTO MODE ###################################################################################################### //

void handle_program_change(){
  
  PROG_BUTTON_STATE = digitalRead(PROG_BUTTON);
  Serial.print("PROG_BUTTON_STATE: ");
  Serial.println(PROG_BUTTON_STATE);
  
  if ( PROG_BUTTON_STATE != PROG_BUTTON_LAST_STATE ) {

    if (PROG_BUTTON_STATE == LOW) {
    
      if (CURRENT_PROGRAM == 1) { CURRENT_PROGRAM = 2; }
      else if (CURRENT_PROGRAM == 2) { CURRENT_PROGRAM = 3; }
      else { CURRENT_PROGRAM = 1; };

      Serial.print("CURRENT PROGRAM: ");
      Serial.println(CURRENT_PROGRAM);
      prog_led(CURRENT_PROGRAM);
    }             
  }

  PROG_BUTTON_LAST_STATE = PROG_BUTTON_STATE;
  
}

void handle_run_button() {
  
  RUN_BUTTON_STATE = digitalRead(RUN_BUTTON);
  Serial.print("RUN_BUTTON_STATE: ");
  Serial.println(RUN_BUTTON_STATE);
  
  if ( RUN_BUTTON_STATE != RUN_BUTTON_LAST_STATE ) {

    if (RUN_BUTTON_STATE == LOW) {
      run_program(CURRENT_PROGRAM);
      Serial.print("AUTO MODE: RUNNING PROGRAM ");
      Serial.println(CURRENT_PROGRAM);
    } 

  }

  RUN_BUTTON_LAST_STATE = RUN_BUTTON_STATE;

}


void run_program(int program) {

  switch(program) {
    case 1:
      servo.write(0);
      for (int i = 7; i < 21; i++) {
        strips.fill(CURRENT_COLOR, i, 1);
        strips.show();
        delay(25);
        strips.clear();
        strips.show();
      }
      servo.write(95);
      break;

    case 2:
      servo.write(0);
      for (int i = 0; i <= 10; i++) {
        for (int j = 7; j < 21; j++) {
          strips.setPixelColor(j, random(0,255), random(0.255), random(0,255), random(5,20));
          strips.show();
          delay(10);
          strips.clear();
          strips.show();
        }       
      }
      servo.write(95);
      break;

    case 3:
      servo.write(0);
      for(int i = 0; i< random(5, 20); i++) {
        strips.setPixelColor(random(7,21), random(0,255), random(0.255), random(0,255), random(2,10));
        strips.show();
        delay(random(20,75));
        strips.clear();
      }
      servo.write(95);
            
      break;
    
  }
  
}

void handle_stop_button() {
  
  STOP_BUTTON_STATE = digitalRead(STOP_BUTTON);
  Serial.print("STOP_BUTTON_STATE: ");
  Serial.println(STOP_BUTTON_STATE);
  
  if ( STOP_BUTTON_STATE != STOP_BUTTON_LAST_STATE ) {

    if (STOP_BUTTON_STATE == LOW) {
      servo.write(95);
      Serial.println("STOPPED");
    }             

  }

  STOP_BUTTON_LAST_STATE = STOP_BUTTON_STATE;

}


// UTILS ######################################################################################################### //
void top_lights() {
  // Pixels: 8 — 21
  strips.fill(CURRENT_COLOR, TOP_BEGIN, TOP_COUNT);
  strips.show();
}


void light_up() {

  pixel.setPixelColor(0  , 255, 0, 0);
  pixel.show();
  pixel.clear();
  pixel.show();
  
  
  digitalWrite(LED_1, HIGH);
  delay(250);
  digitalWrite(LED_1, LOW);
  
  digitalWrite(LED_2, HIGH);
  delay(250);
  digitalWrite(LED_2, LOW);

  digitalWrite(LED_3, HIGH);
  delay(250);
  digitalWrite(LED_3, LOW);

}

void prog_led(int led_number) {
  switch (led_number) {
    
    case 1:
      digitalWrite(LED_1, HIGH);
      digitalWrite(LED_2, LOW);
      digitalWrite(LED_3, LOW);
      break;

    case 2:
      digitalWrite(LED_1, LOW);
      digitalWrite(LED_2, HIGH);
      digitalWrite(LED_3, LOW);
      break;
    
    case 3:
      digitalWrite(LED_1, LOW);
      digitalWrite(LED_2, LOW);
      digitalWrite(LED_3, HIGH);
      break;

    default: 
      digitalWrite(LED_1, LOW);
      digitalWrite(LED_2, LOW);
      digitalWrite(LED_3, LOW);
  }
}
